"""Decorator for Uptime Robot"""
from __future__ import annotations

import asyncio

import aiohttp
import async_timeout

from pyuptimerobot.exceptions import (
    UptimeRobotConnectionException,
    UptimeRobotException,
)

from .const import API_BASE_URL, API_HEADERS, LOGGER
from .models import UptimeRobotApiResponse


def api_request(api_path: str, method: str = "POST"):
    """Decorator for Uptime Robot API request"""

    def decorator(func):
        """Decorator"""

        async def wrapper(*args, **kwargs):
            """Wrapper"""
            client = args[0]
            request_data = f"api_key={client._api_key}&format=json"
            url = f"{API_BASE_URL}{api_path}"
            if kwargs:
                for key, value in kwargs.items():
                    request_data += f"&{key}={value}"
            LOGGER.debug("Requesting %s", url)
            try:
                async with async_timeout.timeout(10, loop=asyncio.get_event_loop()):
                    request = await client._session.request(
                        method=method,
                        url=url,
                        headers=API_HEADERS,
                        data=request_data,
                    )

                    if request.status != 200:
                        raise UptimeRobotConnectionException(
                            f"Request for '{url}' failed with status code '{request.status}'"
                        )

                result = await request.json()
            except aiohttp.ClientError as exception:
                raise UptimeRobotConnectionException(
                    f"Request exception for '{url}' with - {exception}"
                ) from exception

            except asyncio.TimeoutError:
                raise UptimeRobotConnectionException(f"Request timeout for '{url}'")

            except UptimeRobotConnectionException as exception:
                raise UptimeRobotConnectionException(exception) from exception

            except UptimeRobotException as exception:
                raise UptimeRobotException(exception) from exception

            except (Exception, BaseException) as exception:
                raise UptimeRobotException(
                    f"Unexpected exception for '{url}' with - {exception}"
                ) from exception

            LOGGER.debug("Requesting %s returned %s", url, result)

            return UptimeRobotApiResponse.from_dict(
                {**result, "_api_path": api_path, "_method": method}
            )

        return wrapper

    return decorator



"""Uptime Robot models"""
from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Any


class MonitorType(Enum):
    """Monitors type."""

    HTTP = 1
    keyword = 2
    ping = 3


class APIStatus(str, Enum):
    """API status."""

    OK = "ok"
    FAIL = "fail"


class UptimeRobotBaseModel:
    """UptimeRobotBaseModel."""


@dataclass
class UptimeRobotPagination(UptimeRobotBaseModel):
    """Pagination model for Uptime Robot."""

    offset: int = 0
    limit: int = 0
    total: int = 0

    @staticmethod
    def from_dict(data: dict[str, Any]) -> UptimeRobotPagination:
        """Generate object from json."""
        obj: dict[str, Any] = {}
        for key, value in data.items():
            if hasattr(UptimeRobotPagination, key):
                obj[key] = value

        return UptimeRobotPagination(**obj)


@dataclass
class UptimeRobotApiError(UptimeRobotBaseModel):
    """API error model for Uptime Robot."""

    type: str = ""
    parameter_name: str = ""
    message: str = ""
    passed_value: str = ""

    @staticmethod
    def from_dict(data: dict[str, Any]) -> UptimeRobotApiError:
        """Generate object from json."""
        obj: dict[str, Any] = {}
        for key, value in data.items():
            if hasattr(UptimeRobotApiError, key):
                obj[key] = value

        return UptimeRobotApiError(**obj)


@dataclass
class UptimeRobotAccount(UptimeRobotBaseModel):
    """Account model for Uptime Robot."""

    email: str = ""
    up_monitors: int = 0
    down_monitors: int = 0
    paused_monitors: int = 0

    @staticmethod
    def from_dict(data: dict[str, Any]) -> UptimeRobotAccount:
        """Generate object from json."""
        obj: dict[str, Any] = {}
        for key, value in data.items():
            if hasattr(UptimeRobotAccount, key):
                obj[key] = value

        return UptimeRobotAccount(**obj)


@dataclass
class UptimeRobotMonitor(UptimeRobotBaseModel):
    """Monitor model for Uptime Robot."""

    id: int = 0
    friendly_name: str = ""
    url: str = ""
    type: MonitorType = MonitorType.HTTP
    interval: int = 0
    status: int = 0

    @staticmethod
    def from_dict(data: dict[str, Any]) -> UptimeRobotMonitor:
        """Generate object from json."""
        obj: dict[str, Any] = {}
        for key, value in data.items():
            if hasattr(UptimeRobotMonitor, key):
                obj[key] = value

        if obj.get("type"):
            obj["type"] = MonitorType(obj["type"])

        return UptimeRobotMonitor(**obj)


@dataclass
class UptimeRobotApiResponse(UptimeRobotBaseModel):
    """API response model for Uptime Robot."""

    _method: str | None = None
    _api_path: str | None = None
    status: APIStatus = APIStatus.FAIL
    error: UptimeRobotApiError | None = None
    data: list[UptimeRobotMonitor] | UptimeRobotAccount | None = None
    pagination: dict[str, Any] | None = None

    @staticmethod
    def from_dict(data: dict[str, Any]) -> UptimeRobotApiResponse:
        """Generate object from json."""
        obj: dict[str, Any] = {"status": APIStatus(data["stat"])}
        if obj["status"] == APIStatus.FAIL:
            obj["error"] = UptimeRobotApiError.from_dict(data["error"])
        else:
            for key, value in data.items():
                if hasattr(UptimeRobotApiResponse, key):
                    obj[key] = value

            if "pagination" in data:
                obj["pagination"] = UptimeRobotPagination.from_dict(data["pagination"])

            if "monitors" in data:
                obj["data"] = [
                    UptimeRobotMonitor.from_dict(monitor)
                    for monitor in data["monitors"]
                ]
            if "account" in data:
                obj["data"] = UptimeRobotAccount.from_dict(data["account"])

        return UptimeRobotApiResponse(**obj)



"""Uptime Robot constants."""
from logging import Logger, getLogger

LOGGER: Logger = getLogger(__package__)

API_BASE_URL = "https://api.uptimerobot.com/v2"
API_HEADERS = {"Content-Type": "application/x-www-form-urlencoded"}

ATTR_URL = "url"



"""Uptime Robot exceptions."""


class UptimeRobotException(Exception):
    """Base Uptime Robot exception."""


class UptimeRobotConnectionException(UptimeRobotException):
    """Uptime Robot connection exception."""



"""Uptime Robot client."""
from aiohttp import ClientSession

from .decorator import api_request
from .models import UptimeRobotApiResponse


class UptimeRobot:
    """This class is used to get information from Uptime Robot."""

    def __init__(self, api_key: str, session: ClientSession) -> None:
        """Initialize"""
        self._api_key: str = api_key
        self._session: ClientSession = session

    @api_request("/getMonitors")
    async def async_get_monitors(self, **kwargs) -> UptimeRobotApiResponse:
        """Get monitors from API."""

    @api_request("/getAccountDetails")
    async def async_get_account_details(self, **kwargs) -> UptimeRobotApiResponse:
        """Get account details from API."""



"""The setup script."""
from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

setup(
    author_email="hi@ludeeus.dev",
    author="Joakim Sorensen",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    version="21.08.0",
    description="Python API wrapper for Uptime Robot.",
    install_requires=[
        "aiohttp>=3.6.1,<4.0",
        "async_timeout<=3.0.1",
    ],
    keywords=["homeassistant", "version", "update"],
    license="MIT license",
    long_description_content_type="text/markdown",
    long_description=readme,
    name="pyuptimerobot",
    packages=find_packages(include=["pyuptimerobot", "pyuptimerobot*"]),
    python_requires=">=3.8.0",
    url="https://github.com/ludeeus/pyuptimerobot",
)
